package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.TmsTruckFeign;
import com.steerinfo.dil.mapper.QmsQueueListMapper;
import com.steerinfo.dil.model.QmsQueueList;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.model.QmsQueueResult;
import com.steerinfo.dil.mapper.QmsQueueResultMapper;
import com.steerinfo.dil.service.IQmsQueueResultService;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.math.BigDecimal;

/**
 * QmsQueueResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-14 11:27
 * 类描述
 * 修订历史：
 * 日期：2021-09-14
 * 作者：generator
 * 参考：
 * 描述：QmsQueueResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "qmsQueueResultService")
public class QmsQueueResultServiceImpl implements IQmsQueueResultService {

    @Autowired
    private QmsQueueResultMapper qmsQueueResultMapper;

    @Autowired
    private QmsQueueListMapper qmsQueueListMapper;
    
    @Autowired
    private QueuingRulesServiceImpl queuingRulesService;

    @Autowired
    private TmsTruckFeign tmsTruckFeign;

    /**
     * 新增进厂排队实绩
     * @param map {orderNumber}运输订单号
     * @return
     */
    @Override
    public int addQueueResult(Map<String, Object> map) {
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        //生成主键
        qmsQueueResult.setId(qmsQueueResultMapper.selectMaxId());
        //获取传入运输订单号
        String orderNumber = (String) map.get("orderNumber");
        //查询实绩总表ID 运力Id  物资Id (OO-->OOM 一对多)  只有钢材会有多条数据 但都属于钢材类型 取第一个获取物资ID匹配
        map.putAll(qmsQueueResultMapper.queryCIdAndTIdByOrderNumber(orderNumber).get(0)); //使用传进来的map将数据一路带到尾
        //判断是否销售订单 销售订单进厂时查询装车点
        BigDecimal orderType = DataChange.dataToBigDecimal(map.get("orderType"));
        if (orderType.intValue() == 1){
            //查询当前订单关联几种物资
            List<Integer> materialIdList = queuingRulesService.getAllMaterialId(map);
            //取得月台ID、以及对应的物资Id
            List<Map<String, Object>> platIdList = queuingRulesService.getPlatId(materialIdList);
            List<Map<String, Object>> list = new ArrayList<>();
            for (Map<String, Object> map1 : platIdList) {
                map1.put("orderId", map.get("orderId"));
                list.add(map1);
            }
            //远程调用 修改装车点
            tmsTruckFeign.updateLoadResultList(list);
        }
        //获取网格Id
        BigDecimal gridId = queuingRulesService.calculateGridId(DataChange.dataToBigDecimal(map.get("materialId")));
        map.put("gridId", gridId);
        //新增进厂排队实绩 添加申请时间
        qmsQueueResult.setGridId(gridId);
        qmsQueueResult.setResultApplyforTime(new Date());
        //系统自动下发 添加下发时间
        qmsQueueResult.setResultDownTime(new Date());
        //添加运力ID和总实绩ID
        qmsQueueResult.setCapacityId(DataChange.dataToBigDecimal(map.get("capacityId")));
        qmsQueueResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));

        qmsQueueResult.setInsertTime(new Date());
        qmsQueueResult.setInsertUsername("admin");
        return qmsQueueResultMapper.insertSelective(qmsQueueResult);
    }


    /**
     * 新增仓库排队实绩
     * @param map orderNumber运输订单号
     * @return
     */
    public int addQueueResultByCk(Map<String, Object> map){
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        //生成主键
        qmsQueueResult.setId(qmsQueueResultMapper.selectMaxId());
        //查询当前订单下所有的装车作业点
        List<Integer> loadingIdList = qmsQueueResultMapper.getAllLoadingByOrderNumber((String) map.get("orderNumber"));
        //遍历所有装车点查询排队车辆数 选择最少的一个先进行排队
        int platId = 0;
        int num = 100; //
        for (Integer platformId : loadingIdList) {
            int count = queuingRulesService.queueCountByPlatformId(new BigDecimal(platformId)).intValue();
            if(count == 0){
                platId = platformId;
                break;
            }
            if(count < num){
                num = count; //取代比较数
                platId = platformId; //替换月台ID
            }
        }
        //获取添加网格Id
        map.put("queueLocationId", platId);
        map.put("gridType", 1);
        qmsQueueResult.setGridId(queuingRulesService.getGridIdByGatepostIdAndGridType(map));
        //通过运输订单号查询实绩ID 等数据
        map.putAll(qmsQueueResultMapper.queryCIdAndTIdByOrderNumber((String) map.get("orderNumber")).get(0));
        qmsQueueResult.setResultApplyforTime(new Date());
        //系统自动下发 添加下发时间
        qmsQueueResult.setResultDownTime(new Date());
        //添加运力ID和总实绩ID
        qmsQueueResult.setCapacityId(DataChange.dataToBigDecimal(map.get("capacityId")));
        qmsQueueResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        qmsQueueResult.setInsertTime(new Date());
        qmsQueueResult.setInsertUsername("admin");
        return qmsQueueResultMapper.insertSelective(qmsQueueResult);
    }

    /**
     * 查看排队转移界面
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getChangeQueueMes(Map<String, Object> map) {
        return qmsQueueResultMapper.getChangeQueueMes(map);
    }

    /**
     * 查询排队申请
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueApply(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueApply(map);
    }

    /**
     * 查询执行中运单
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllExecutionOrder(Map<String, Object> map) {
        return qmsQueueResultMapper.getAllExecutionOrder(map);
    }

    /**
     * 指令接收 判断条件：是否有排队结束时间
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> orderReceive(Map<String, Object> map) {
        Integer queueType = (Integer) map.get("queueType");
        List<Map<String, Object>> orderReceiveList = null;
        switch (queueType){
            case 1: //进厂
                orderReceiveList = qmsQueueResultMapper.getOrderReceive(map);
                break;
            case 2: //仓库
                orderReceiveList = qmsQueueResultMapper.getWarehouseOrderReceive(map);
        }
        //遍历数据 通过是否有接收时间 来判断是否接收
        for (Map<String, Object> orderReceiveMap : orderReceiveList) {
            if(orderReceiveMap.get("resultAcceptTime") != null){
                orderReceiveMap.put("isReceive", "是");
            }else {
                orderReceiveMap.put("isReceive", "否");
            }
        }
        return orderReceiveList;
    }

    /**
     * 查询排队开始 数据
     * 可用 gridId 网格ID
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueListByQueueUp(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueListByQueueUp(map);
    }

    /**
     * 查询排队详情
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueMes(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueMes(map);
    }

    /**
     * 查询排队取消数据
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueCancel(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueCancel(map);
    }

    /**
     * 通过实绩ID查询排队取消信息
     * @param resultId
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueCancelByResultId(Integer resultId) {
        return qmsQueueResultMapper.getQueueCancelByResultId(resultId);
    }

    /**
     * 修改排队取消实绩
     * @param qmsQueueResult
     * @return
     */
    @Override
    public int updateQueueCancel(QmsQueueResult qmsQueueResult) {
        qmsQueueResult.setUpdateTime(new Date());
        qmsQueueResult.setUpdateUsername("admin");
        return qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
    }

    /**
     * 查询排队插队
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueInsert(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueInsert(map);
    }

    /**
     * 查询排队结束
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueEndResult(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueEndResult(map);
    }

    /**
     * 汽车监控
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> capacityMonitor(Map<String, Object> map) {
        List<Map<String, Object>> list = qmsQueueResultMapper.capacityMonitor(map);
        for (Map<String, Object> capacityMap : list) {
            //如果结束时间为空 排队总时长为
            if(capacityMap.get("resultEndTime") == null){
                Date date1 = (Date) capacityMap.get("resultApplyforTime");
                Date date2 = new Date();
                String differenceTime = DataChange.calculatedTimeDifference(date1, date2);
                capacityMap.put("resultQueueAllTime",differenceTime);
            }else {
                //否则排队时长为结束时长 - 申请时长
                Date date1 = (Date)capacityMap.get("resultApplyforTime");
                Date date2 = (Date)capacityMap.get("resultEndTime");
                String differenceTime = DataChange.calculatedTimeDifference(date1, date2);
                capacityMap.put("resultQueueAllTime",differenceTime);
            }
        }
        return list;
    }

    //=========================================仓库排队===========================================

    /**
     * 查看仓库排队申请
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getWarehouseQueueApply(Map<String, Object> map) {
        return qmsQueueResultMapper.getWarehouseQueueApply(map);
    }

    /**
     * 查看排队开始
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getWarehouseQueueListByQueueUp(Map<String, Object> map) {
        return qmsQueueResultMapper.getWarehouseQueueListByQueueUp(map);
    }

    /**
     * 查看仓库排队详情 通过运输订单号查询仓库
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getWarehouseQueueMes(Map<String, Object> map) {
        return qmsQueueResultMapper.getWarehouseQueueMes(map);
    }

    /**
     * 查询仓库排队转移详情
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getWarehouseQueueChange(Map<String, Object> map) {
        return qmsQueueResultMapper.getWarehouseQueueChange(map);
    }

    /**
     * 查询仓库排队结束
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getWarehouseQueueEndMes(Map<String, Object> map) {
        return qmsQueueResultMapper.getWarehouseQueueEndMes(map);
    }

    //********************************APP*********************************************

    /**
     * 根据车牌号查询排队下发
     * @param capacityNumber
     * @return
     */
    @Override
    public List<Map<String, Object>> getOrderReceive2(String capacityNumber) {
        BigDecimal gridId = getGridId(capacityNumber);
        if (gridId == null) {
            List<Map<String,Object>> mapList = new ArrayList<>();
            return mapList;
        }
        Map<String,Object> map = new HashMap<>();
        map.put("capacityNumber",capacityNumber);
        map.put("gridId",gridId);
        List<Map<String, Object>> orderReceive = qmsQueueResultMapper.getOrderReceive2(map);
        return orderReceive;
    }

    /**
     * 排队取消
     * @param mapValue
     * @return
     */
    @Override
    public int cancelQueue(Map<String,Object> mapValue) {
        String capacityNumber = (String) mapValue.get("capacityNumber");
        String resultCancelReason = (String) mapValue.get("resultCancelReason");
        BigDecimal resultId = qmsQueueResultMapper.getResultIdByCapacityId(capacityNumber);
        QmsQueueResult qmsQueueResult = qmsQueueResultMapper.selectByPrimaryKey(resultId);
        Map<String, Object> listIdMap = qmsQueueResultMapper.getListIdByResultId(resultId);
        BigDecimal listId = (BigDecimal) listIdMap.get("listId");
        QmsQueueList qmsQueueList = qmsQueueListMapper.selectByPrimaryKey(listId);
        qmsQueueList.setDeleted(new BigDecimal(1));
        qmsQueueResult.setResultCancelTime(new Date());
        qmsQueueResult.setResultCancelReason(resultCancelReason);
        qmsQueueListMapper.updateByPrimaryKeySelective(qmsQueueList);
        return qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
    }


    /**
     * 通过车牌号得到网格id
     * @param capacityNumber
     * @return
     */
    @Override
    public BigDecimal getGridId(String capacityNumber) {
        return qmsQueueResultMapper.getGridIdNotNull(capacityNumber);
    }


    /**
     * 查询各门岗和仓库的排队状态
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueStatusList() {
        return qmsQueueResultMapper.getQueueStatusList();
    }


}
